/*

Database Frame That Displays Tables And Fields

Author :	Dattaraj J. Rao
			jagrao@goa1.dot.net.in

*/

import java.awt.*;
import java.awt.event.*;
import java.sql.*;

class DBaseFrame extends Frame implements WindowListener,ItemListener,ActionListener,KeyListener
{
	MenuItem runquery,newquery,savequery,loadquery,exit;

	List tables,columns;
	TextArea sql;
	Label tabs,cols,qry;

	GridBagLayout grd;
	GridBagConstraints gbc;

	String datasource;
	Connection conn;
	ResultSet results;
	ResultSetMetaData rma;

	Results out;
	Alert alert;
	LSDialog dialog;

	public DBaseFrame()
	{
	}

	public DBaseFrame(String title,Connection connection)
	{
		super(title);
		addWindowListener(this);
		setBounds(50,5,700,500);

		conn=connection;

		MenuBar mbr=new MenuBar();
		setMenuBar(mbr);

		// Query Menu
		Menu query=new Menu("Query");

		runquery=new MenuItem("Run Query",new MenuShortcut('R'));
		runquery.addActionListener(this);
		query.add(runquery);

		newquery=new MenuItem("New Query",new MenuShortcut('N'));
		newquery.addActionListener(this);
		query.add(newquery);

		mbr.add(query);

		// File Menu
		Menu file=new Menu("File");

		savequery=new MenuItem("Save Query",new MenuShortcut('S'));
		savequery.addActionListener(this);
		file.add(savequery);

		loadquery=new MenuItem("Load Query",new MenuShortcut('L'));
		loadquery.addActionListener(this);
		file.add(loadquery);

		exit=new MenuItem("Quit",new MenuShortcut('Q'));
		exit.addActionListener(this);
		file.add(exit);

		mbr.add(file);
		
		grd=new GridBagLayout();
		setLayout(grd);
		gbc=new GridBagConstraints();

		Font fl=new Font("",Font.BOLD,15);

		tabs=new Label("Tables",Label.CENTER);
		tabs.setFont(fl);
		cols=new Label("Columns",Label.CENTER);
		cols.setFont(fl);
		qry=new Label("SQL Statements",Label.CENTER);
		qry.setFont(fl);

		tables=new List(10);
		columns=new List(10);
		sql=new TextArea();
		sql.addKeyListener(this);
		sql.setFont(new Font("",Font.PLAIN,15));

		tables.addItemListener(this);
		columns.addItemListener(this);

		addComponent(tabs,0,0,50,10);
		addComponent(cols,50,0,100,10);
		addComponent(qry,0,120,50,10);

		addComponent(tables,0,10,50,100);
		addComponent(columns,50,10,50,100);
		addComponent(sql,0,140,100,100);

		setTables();
		setColumns();

		out=new Results("Query Results");

		alert=new Alert(this,"Error",true);

		dialog=new LSDialog(this,sql);

		Image icon=getToolkit().getImage("icon.gif");
		setIconImage(icon);

		setVisible(true);
		sql.requestFocus();
	}
	
	void addComponent(Component c,int x,int y,int w,int h)
	{
		gbc.gridx=x;
		gbc.gridy=y;
		gbc.gridwidth=w;
		gbc.gridheight=h;
		gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.insets.right=10;
		gbc.insets.left=10;
		gbc.insets.top=2;
		gbc.insets.bottom=2;
		add(c);
		grd.setConstraints(c,gbc);
	}

	void setTables()
	{
		try
		{
			DatabaseMetaData dma=conn.getMetaData();
			String type[]=new String[1];
			type[0]="TABLES";
			results=dma.getTables(null,null,"%",type);

			setData(tables,results,"TABLE_NAME");
			results.close();
		}
		catch(SQLException exp1)
		{
			alert.doAlert("Error In Connecting To DataBase.");
		}
	}
	
	void setColumns()
	{
		try
		{
			String table=tables.getSelectedItem();
			String query="SELECT * FROM ["+table+"]";
			if(runQuery(query))
				setData(columns,results);
			results.close();
		}
		catch(SQLException exp1)
		{
			alert.doAlert("Error In Connecting To DataBase.");
		}
	}

// For setting coulmns of a resultset in a list
	void setData(List list,ResultSet result)
	{
		list.removeAll();
		try
		{
			ResultSetMetaData rma=results.getMetaData();
			int cols=rma.getColumnCount();
			for(int i=1;i<=cols;i++)
				list.add(rma.getColumnName(i));
		}
		catch(SQLException exp1)
		{
			//alert.doAlert("Error In Connecting To DataBase.");
		}
	}		

//For setting data from a column of resultset in a list
	void setData(List list,ResultSet result,String column)
	{
		list.removeAll();
		try
		{
			while(results.next())
				list.add(results.getString(column));
			list.select(0);
		}
		catch(SQLException exp1)
		{
			//alert.doAlert("Error In Connecting To DataBase");
		}
	}		

// Run a query in resultset
	boolean runQuery(String query)
	{
		try
		{
			Statement smt=conn.createStatement();
			results=smt.executeQuery(query);
			return true;
		}
		catch(SQLException exp1)
		{
			alert.doAlert("Error In SQL Statement");
			return false;
		}
	}

	public void windowClosing(WindowEvent e)
	{
		try
		{
			conn.close();
		}
		catch(Exception exp1)
		{
		}
		System.exit(0);
	}

	public void windowClosed(WindowEvent e){ }
	public void windowOpened(WindowEvent e){ }
	public void windowIconified(WindowEvent e){ }
	public void windowDeiconified(WindowEvent e){ }
	public void windowActivated(WindowEvent e){ }
	public void windowDeactivated(WindowEvent e){ }

	public void itemStateChanged(ItemEvent e)
	{
		if(e.getStateChange()==ItemEvent.SELECTED && e.getSource()==tables)
			setColumns();
		sql.requestFocus();
	}

	public void actionPerformed(ActionEvent e)
	{
		if(e.getSource()==runquery)
		{
			if(runQuery(sql.getText()))
				out.showResults(results);
		}

		if(e.getSource()==newquery)
		{
			sql.setText("");
			sql.requestFocus();
		}

		if(e.getSource()==loadquery)
			dialog.load("Load SQL Query","sql");

		if(e.getSource()==savequery)
		{
			dialog.save("Save SQL Query","sql");
		}

		if(e.getSource()==exit)
		{
			try
			{
				conn.close();
			}
			catch(Exception exp1)
			{
			}
			System.exit(0);
		}
	}

	public void keyPressed(KeyEvent e)
	{
		if(e.getKeyCode()==KeyEvent.VK_SEMICOLON && runQuery(sql.getText()))
			out.showResults(results);
	}

	public void keyReleased(KeyEvent e){ }
	public void keyTyped(KeyEvent e){ }

}